#include <stdlib.h>

#include <arch.h>
#include <device.h>

#include <ggi/ggi.h>

/* Externally used variables: */
byte *HW_OffscreenBuffer=0;
signed_32bit HW_ScreenSizeX;
signed_32bit HW_ScreenSizeY;
signed_32bit HW_ScreenMaxX;
signed_32bit HW_ScreenMaxY;
signed_32bit HW_ScreenCenterX;
signed_32bit HW_ScreenCenterY;
unsigned_32bit HW_BytesPerRow;

/* Internal variables: */

static ggi_visual_t disp = NULL;
static const ggi_directbuffer	*dbuf;
static int page_flipping_on;
static int visible_screen;
static int vars_set=0;
static int accel_clear=0;

void HW_libggi_setvars(int x,int y, int page_flip,int accel_clr)
{
  vars_set=1;
  HW_ScreenSizeX=x;
  HW_ScreenSizeY=y;
  page_flipping_on = page_flip;
  accel_clear = accel_clr;
}

int HW_OpenScreen(void)
{
  int err;
  ggi_mode mode;

  if (!vars_set) {
    HW_ScreenSizeX = GGI_AUTO;
    HW_ScreenSizeY = GGI_AUTO;
    page_flipping_on = 0;
  }

  /* do some hardware initalization */

  ggiInit();
  
  if ( !(disp = ggiOpen (NULL)) )
    return 0;

  err = ggiCheckGraphMode (disp, HW_ScreenSizeX, HW_ScreenSizeY,
                           GGI_AUTO, GGI_AUTO, GT_8BIT, &mode);
  if (err) {
    ggiClose (disp);
    ggiExit ();
    fprintf (stderr, "Couldn't get an 8bpp display.\n");
    return 0;
  }

#if 0
  mode.frames = 2;              /* use this instead of direct blitting? */
#endif

  HW_ScreenSizeX = mode.visible.x;
  HW_ScreenSizeY = mode.visible.y;
  mode.virt.x = HW_ScreenSizeX;
  mode.virt.y = (page_flipping_on)?HW_ScreenSizeY*2:HW_ScreenSizeY;

  HW_ScreenMaxX=HW_ScreenSizeX-1;
  HW_ScreenMaxY=HW_ScreenSizeY-1;
  HW_ScreenCenterX=(HW_ScreenSizeX/2)<<4;
  HW_ScreenCenterY=(HW_ScreenSizeY/2)<<4;

  err = ggiSetMode (disp, &mode);
  if (err) {
    ggiClose (disp);
    ggiExit ();
    return 0;
  }
  ggiSetFlags(disp, GGIFLAG_ASYNC);

  dbuf = ggiDBGetBuffer (disp, 0);

  if (!dbuf)
    return 0;
#if 0
  if (ggiDBGetLayout (dbuf) != blPixelLinearBuffer)
    return 0;
  if ( !(plb = ggiDBGetPLB (dbuf)) )
    return 0;
#endif

  if (page_flipping_on) {
    HW_BytesPerRow = dbuf->buffer.plb.stride;
    ggiSetOrigin(disp, 0, 0);
    visible_screen = 0;
    HW_OffscreenBuffer = (byte *)dbuf->write + HW_BytesPerRow*HW_ScreenSizeY;
  } else {
    HW_BytesPerRow = HW_ScreenSizeX;
    HW_OffscreenBuffer = (byte *)malloc(HW_BytesPerRow*HW_ScreenSizeY);
  } 
  return 1;
}

void HW_DisplayScreen(void)
{
  if (page_flipping_on) {
    if (visible_screen==1) {
      ggiSetOrigin(disp, 0, 0);
      visible_screen = 0;
      HW_OffscreenBuffer = (byte *)dbuf->write + HW_BytesPerRow*HW_ScreenSizeY;
    } else {
      ggiSetOrigin(disp, 0, HW_ScreenSizeY);
      visible_screen = 1;
      HW_OffscreenBuffer = (byte *)dbuf->write;
    }
  } else {
    int y;
    byte *src,*dst;
    src = HW_OffscreenBuffer;
    dst = (byte *)dbuf->write;
    
    for (y = 0; y < HW_ScreenSizeY; y++) {
      memcpy (dst, src, HW_ScreenSizeX);
      src += HW_ScreenSizeX;
      dst += dbuf->buffer.plb.stride;
    }
  }
  ggiFlush(disp);
  /* wait for horizontal retrace here! */
}

void HW_SetPalette(byte *orgpal) 
{
  int i;
  ggi_color pal[256];

  for (i=0;i<256;i++) {
    pal[i].r = orgpal[i*3]<<8;
    pal[i].g = orgpal[i*3+1]<<8;
    pal[i].b = orgpal[i*3+2]<<8;
  }
  
  ggiSetPalette (disp, 0, 256, pal);
}

void HW_ClearOffscreen(void)
{
    if ( (page_flipping_on == 1) && accel_clear ) 
      /* ggiSetGCForeground ... unimplemented */
      ggiDrawBox(disp, 0, visible_screen == 1 ? 0 : HW_ScreenSizeY, HW_ScreenSizeX, HW_ScreenSizeY);
    else
    {
      HW_MemSet(HW_OffscreenBuffer, 0, HW_BytesPerRow*HW_ScreenSizeY);
    }
}

void HW_CloseScreen(void)
{
  /* close screen */

  ggiClose(disp);
}


int HW_KeyboardHit(void)
{
  return ggiKbhit(disp);
}

int HW_GetKey(void)
{
  int c;
  c=ggiGetc(disp);
  switch(GII_KTYP(c)) {
  case GII_KT_LATIN1:
  case GII_KT_CONS:
    return GII_KVAL(c);
  default:
    switch(c) {
    case GIIK_P0: return HW_KEY_KP0; 
    case GIIK_P1: return HW_KEY_KP1; 
    case GIIK_P2: return HW_KEY_KP2; 
    case GIIK_P3: return HW_KEY_KP3; 
    case GIIK_P4: return HW_KEY_KP4; 
    case GIIK_P5: return HW_KEY_KP5; 
    case GIIK_P6: return HW_KEY_KP6; 
    case GIIK_P7: return HW_KEY_KP7; 
    case GIIK_P8: return HW_KEY_KP8; 
    case GIIK_P9: return HW_KEY_KP9; 
#if 0
    case GIIK_Home: return HW_KEY_HOME;
    case GIIK_End: return HW_KEY_END;
#endif
    case GIIK_Up: return HW_KEY_UP;
    case GIIK_Down: return HW_KEY_DOWN;
    case GIIK_Left: return HW_KEY_LEFT;
    case GIIK_Right: return HW_KEY_RIGHT;
    default: return 0;
    }
  }
}

