#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include <arch.h>
#include <error.h>
#include "vector.h"
#include <object.h>
#include "readobj.h"

ObjectDesc *ReadGEM(FILE *fil,double scaledsize,int shade)
{
  int numvert,numpolys,totalpolyvert;
  double x,y,z;
  int i,j,n,polnr,w;
  ObjectDesc *desc;
  int polyvert[MAXVERTINPOLY];
  real r,rmax;
  double x1,x2,y1,y2,z1,z2;
  double xn,yn,zn,norm;
  double *fvertices;

  desc=malloc(sizeof(ObjectDesc));
  assert(desc!=0);
  
  fscanf(fil,"%d %d %d",&numvert,&numpolys,&totalpolyvert);

  desc->NumVertices=numvert;
  desc->Vertex=malloc(sizeof(vector)*numvert);
  assert(desc->Vertex!=0);
  fvertices=malloc(sizeof(double)*3*numvert);
  assert(fvertices!=0);

  rmax=0;
  for (i=0;i<numvert;i++) {
    fscanf(fil,"%lf %lf %lf",&x,&y,&z);
    fvertices[3*i]=x;
    fvertices[3*i+1]=y;
    fvertices[3*i+2]=z;

    r=sqrt((x*x+y*y+z*z));

    if (r>rmax)
	rmax=r;

  }
  desc->Radius=rmax*scaledsize/rmax;

  for (i=0;i<numvert;i++) {
    desc->Vertex[i].x=fvertices[3*i]*scaledsize/rmax;
    desc->Vertex[i].y=fvertices[3*i+1]*scaledsize/rmax;
    desc->Vertex[i].z=fvertices[3*i+2]*scaledsize/rmax;
  }
  

  desc->Poly=malloc(sizeof(Polygon)*numpolys);
  assert(desc->Poly!=0);

  for (polnr=0;polnr<numpolys;polnr++) {
    fscanf(fil,"%d",&numvert);
    assert(numvert<=MAXVERTINPOLY-1);
    desc->Poly[polnr].NumVertices=numvert;
    for (j=0;j<numvert;j++) {
      fscanf(fil,"%d",&n);
      n--;
      polyvert[j]=n;
      desc->Poly[polnr].Vertex[j]=n;
    }
    if (shade)
      desc->Poly[polnr].Type=POLY_GOROUD;
    else
      desc->Poly[polnr].Type=POLY_FLAT;
    desc->Poly[polnr].Color=1;
    desc->Poly[polnr].BackFaceCulled=1;

    x1=fvertices[polyvert[1]*3+0] - fvertices[polyvert[0]*3+0];
    y1=fvertices[polyvert[1]*3+1] - fvertices[polyvert[0]*3+1];
    z1=fvertices[polyvert[1]*3+2] - fvertices[polyvert[0]*3+2];
    x2=fvertices[polyvert[2]*3+0] - fvertices[polyvert[0]*3+0];
    y2=fvertices[polyvert[2]*3+1] - fvertices[polyvert[0]*3+1];
    z2=fvertices[polyvert[2]*3+2] - fvertices[polyvert[0]*3+2];

    xn=y1*z2 - y2*z1;
    yn=z1*x2 - z2*x1;
    zn=x1*y2 - y1*x2;
    norm=xn*xn + yn*yn + zn*zn;
    norm=sqrt(norm);
    if (norm>0.000001) {
      desc->Poly[polnr].Normal.x=-xn/norm;
      desc->Poly[polnr].Normal.y=-yn/norm;
      desc->Poly[polnr].Normal.z=-zn/norm;
    } else {
      desc->Poly[polnr].Normal.x=1;
      desc->Poly[polnr].Normal.y=0;
      desc->Poly[polnr].Normal.z=0;
    }
  }
  
  desc->NumPolys=numpolys;
  
  desc->VertexNormal=malloc(sizeof(vector) * desc->NumVertices);
  assert(desc->VertexNormal!=0);
  if (!shade) { /* flat */
  } else { /* shaded */
    for (i=0;i<desc->NumVertices;i++) {
      xn=0;
      yn=0;
      zn=0;
      n=0;
      for (j=0;j<desc->NumPolys;j++) {
	for (w=0;w<desc->Poly[j].NumVertices;w++) {
	  if (desc->Poly[j].Vertex[w]==i) {
	    xn+=desc->Poly[j].Normal.x;
	    yn+=desc->Poly[j].Normal.y;
	    zn+=desc->Poly[j].Normal.z;
	    n++;
	    break;
	  }
	}
      }
      if (n!=0) {
	desc->VertexNormal[i].x=xn/n;
	desc->VertexNormal[i].y=yn/n;
	desc->VertexNormal[i].z=zn/n;
      }
    }
  }

  return desc;
}

ObjectDesc *ReadGEO(FILE *fil,double scaledsize,int shade)
{
  int numvert;
  double x,y,z;
  int i,j,n,polnr,w;
  ObjectDesc *desc;
  int polyvert[MAXVERTINPOLY];
  real r,rmax;
  double x1,x2,y1,y2,z1,z2;
  double xn,yn,zn,norm;
  double *fvertices;
  int color,tmp,tmp2;
  int res;

  desc=malloc(sizeof(ObjectDesc));
  assert(desc!=0);
  
  fscanf(fil,"3DG1 %d",&numvert);

  desc->NumVertices=numvert;
  desc->Vertex=malloc(sizeof(vector)*numvert);
  assert(desc->Vertex!=0);
  fvertices=malloc(sizeof(double)*3*numvert);
  assert(fvertices!=0);
  rmax=0;
  for (i=0;i<numvert;i++) {
    fscanf(fil,"%lf %lf %lf",&x,&y,&z);
    fvertices[3*i]=x;
    fvertices[3*i+1]=y;
    fvertices[3*i+2]=z;

    r=sqrt((x*x+y*y+z*z));

    if (r>rmax)
	rmax=r;

  }

  desc->Radius=rmax*scaledsize/rmax;

  for (i=0;i<numvert;i++) {
    desc->Vertex[i].x=fvertices[3*i]*scaledsize/rmax;
    desc->Vertex[i].y=fvertices[3*i+1]*scaledsize/rmax;
    desc->Vertex[i].z=fvertices[3*i+2]*scaledsize/rmax;
  }

  desc->Poly=malloc(sizeof(Polygon)*MAXPOLYGONS);
  assert(desc->Poly!=0);
  polnr=0;
  while (1) {
    res=fscanf(fil,"%d",&numvert);
    if ((res==0)||(res==EOF))  {
      break;
    }
    assert(numvert<=MAXVERTINPOLY-1);
    for (j=0;j<numvert;j++) {
      fscanf(fil,"%d",&polyvert[j]);
    }
    res=fscanf(fil,"%d",&color);
    if ((res==0)||(res==EOF)) 
      break;
    if (color<0) {
      color=-color;
      fscanf(fil,"%d",&tmp);
      for (i=0;i<tmp;i++) {
        fscanf(fil,"%d",&n);
	n++;
 	for (j=0;j<n;j++) {
          fscanf(fil,"%d",&tmp2);
        }
      }
    }

    if (shade)
      desc->Poly[polnr].Type=POLY_GOROUD;
    else
      desc->Poly[polnr].Type=POLY_FLAT;
    desc->Poly[polnr].Color=color%16;
    desc->Poly[polnr].BackFaceCulled=1;
    desc->Poly[polnr].NumVertices=numvert;
    for (j=0;j<numvert;j++) {
      desc->Poly[polnr].Vertex[numvert-1-j]=polyvert[j];
    }
    x1=fvertices[polyvert[1]*3+0] - fvertices[polyvert[0]*3+0];
    y1=fvertices[polyvert[1]*3+1] - fvertices[polyvert[0]*3+1];
    z1=fvertices[polyvert[1]*3+2] - fvertices[polyvert[0]*3+2];
    x2=fvertices[polyvert[2]*3+0] - fvertices[polyvert[0]*3+0];
    y2=fvertices[polyvert[2]*3+1] - fvertices[polyvert[0]*3+1];
    z2=fvertices[polyvert[2]*3+2] - fvertices[polyvert[0]*3+2];

    xn=y1*z2 - y2*z1;
    yn=z1*x2 - z2*x1;
    zn=x1*y2 - y1*x2;
    norm=xn*xn + yn*yn + zn*zn;
    norm=sqrt(norm);
    if (norm>0.00000001) {
      desc->Poly[polnr].Normal.x=xn/norm;
      desc->Poly[polnr].Normal.y=yn/norm;
      desc->Poly[polnr].Normal.z=zn/norm;
    } else {
      desc->Poly[polnr].Normal.x=1;
      desc->Poly[polnr].Normal.y=0;
      desc->Poly[polnr].Normal.z=0;
    }
    polnr++;
  }
  
  desc->NumPolys=polnr;
  desc->Poly=realloc(desc->Poly,sizeof(Polygon)*polnr);

  desc->VertexNormal=malloc(sizeof(vector)*desc->NumVertices);
  assert(desc->VertexNormal!=0);
  if (!shade) { /* flat */
  } else { /* shaded */
    for (i=0;i<desc->NumVertices;i++) {
      xn=0;
      yn=0;
      zn=0;
      n=0;
      for (j=0;j<desc->NumPolys;j++) {
	for (w=0;w<desc->Poly[j].NumVertices;w++) {
	  if (desc->Poly[j].Vertex[w]==i) {
	    xn+=desc->Poly[j].Normal.x;
	    yn+=desc->Poly[j].Normal.y;
	    zn+=desc->Poly[j].Normal.z;
	    n++;
	    break;
	  }
	}
      }
      if (n!=0) {
	desc->VertexNormal[i].x=xn/n;
	desc->VertexNormal[i].y=yn/n;
	desc->VertexNormal[i].z=zn/n;
      }
    }
  }

  return desc;
}
